function out=simdaq(arg1,arg2,flag,fcnname,B)
% SIMDAQ  Simulate signal filtering, sampling, and spectral analysis.
% Signal functions are used as in Sampling().   Four signal functions
% are supplied; see the HELP file aboutsimdaq.m for specifics.
% A low-pass filter can be selected in Filter Design mode, to apply
% to the signal before sampling, and a data window can be selected in
% Window mode, to apply to sampled data before spectral analysis.
% The spectral analysis results can be returned to the main workspace.
%
% SIMDAQ('myfun','F') replaces the built-in signal UNITSQUARE with
% the user-defined function F, which is of the form y=F(t).

if nargin==5	% ODEFile functionality
	% arg1 is t
	% arg2 is r, the vector of y and its derivatives
	F=feval(fcnname,arg1);		% forcing function
	% out is rprime, the time-derivative of r
	out=[arg2(2:end);B*[arg2;F]];
	% coefficient array B is pre-defined to make this operation
	% as fast as possible
	
elseif (nargin==1 & isstr(arg1))	% GUI callback functionality
	action=arg1;
	f=get(gcbf,'UserData');
	switch (action)
	case 'popmode'
		switch f.mode
		case 1			% Exit Signal Mode
			axes(f.datax),		legend off
			set(get(f.datax,'Children'),'Visible','off')
			axes(f.spectax),	legend off,	cla
			set(f.daqclump,'Visible','off')
		case 2			% Exit Filter Mode
			axes(f.magax),		cla
			axes(f.phaseax),	cla
			axes(f.poleax),		cla
			set(f.filtclump,'Visible','off')
		case 3			% Exit Window Mode
			axes(f.magax),		cla
			axes(f.phaseax),	cla
			axes(f.poleax),		cla
			set(f.winclump,'Visible','off')
		end
		f.mode=get(gcbo,'Value');
		switch f.mode
		case 1			% Enter Signal Mode
			set(f.daqclump,'Visible','on')
			[f.y,f.Y,f.mean]=daqdisp(f);
			f.newflag=0;
		case 2			% Enter Filter Mode
			set(f.filtclump,'Visible','on')
			f.C=filterc(f.filter,f.order,f.fc,f.ripple);
			f.dots=filtdisp(f);
		case 3			% Enter Window Mode
			set(f.winclump,'Visible','on')
			windisp(f)
		end
	case 'popsig'
		f.signal=get(gcbo,'Value');
		f.newflag=1;
		[f.y,f.Y,f.mean]=daqdisp(f);
		f.newflag=0;
	case 'fs'
		f.fs=max([realmin^0.7,str2num(get(gcbo,'String'))]);
		set(gcbo,'String',num2str(f.fs))
		f.newflag=1;
		[f.y,f.Y,f.mean]=daqdisp(f);
		f.newflag=0;
	case 'N'
		f.N=max([2,round(str2num(get(gcbo,'String')))]);
		set(gcbo,'String',num2str(f.N))
		f.newflag=1;
		[f.y,f.Y,f.mean]=daqdisp(f);
		f.newflag=0;
	case 'start'
		f.start=str2num(get(gcbo,'String'));
		set(gcbo,'String',num2str(f.start));
		f.newflag=1;
		[f.y,f.Y,f.mean]=daqdisp(f);
		f.newflag=0;
	case 'warmup'
		f.warmup=abs(str2num(get(gcbo,'String')));
		set(gcbo,'String',num2str(f.warmup));
		f.newflag=1;
		[f.y,f.Y,f.mean]=daqdisp(f);
		f.newflag=0;
	case 'popdisp'
		f.spectmode=get(gcbo,'Value');
		daqdisp(f);
	case 'zoom'
		if (get(gcbo,'Value'))
			axes(f.datax),		zoom on
			axes(f.spectax),	zoom on
		else
			axes(f.datax),		zoom out,	zoom off, axis auto
			axes(f.spectax),	zoom out,	zoom off, axis auto
		end
	case 'return'
		t=f.start+[0:f.N-1]'/f.fs;
		assignin('base','tout',t)
		fcnnames={f.fcnname;'unitstep';'problem';'noisy'};
		F=feval(fcnnames{f.signal},t);
		assignin('base','Fout',F)
		assignin('base','yout',f.y)
		disp('Returned filter output to [tout,Fout,yout]')
		if f.spectmode<5
			if mod(f.N,2)	mid=(f.N+1)/2; else mid=f.N/2+1;	end
			n=1:mid;
			freq=[0:mid-1]'/f.N*f.fs;
			assignin('base','freq',freq)
		end
		switch(f.spectmode)
		case 1
			assignin('base','AY',2*abs(f.Y(n))/f.N)
			disp('Returned Amplitude Spectrum to [freq,AY]')
		case 2
			assignin('base','PY',2*f.Y(n).*conj(f.Y(n))/f.N^2)
			disp('Returned Amplitude Spectrum to [freq,AY]')
		case 3
			assignin('base','CY',2*f.Y(n)/f.N)
			disp('Returned Complex Spectrum to [freq,CY]')
		case 4
			assignin('base','dBY',20*log10(sqrt(2)*abs(f.Y(n))/f.N))
			disp('Returned Decibel Spectrum to [freq,dBY]')
		case 5
			assignin('base','ydat',real(ifft(f.Y)))
			disp('Returned Windowed Sample Data to ydat')
		end
	case 'popfilt'
		f.filter=get(gcbo,'Value');
		if f.filter==4
			set(f.filtclump(10),'Enable','on')
		else
			set(f.filtclump(10),'Enable','off')
		end
		f.C=filterc(f.filter,f.order,f.fc,f.ripple);
		f.dots=filtdisp(f);
		f.newflag=1;
	case 'order'
		f.order=max([1,round(str2num(get(gcbo,'String')))]);
		set(gcbo,'String',num2str(f.order))
		f.C=filterc(f.filter,f.order,f.fc,f.ripple);
		f.dots=filtdisp(f);
		f.newflag=1;
	case 'fc'
		v=str2num(get(gcbo,'String'));
		f.fc=min([realmax^0.7,max([realmin^0.7,v])]);
		set(gcbo,'String',num2str(f.fc))
		f.C=filterc(f.filter,f.order,f.fc,f.ripple);
		f.dots=filtdisp(f);
		f.newflag=1;
	case 'ripple'
		crit=1-sqrt(1/(1+eps^0.7));
		f.ripple=min([0.999,max([crit,str2num(get(gcbo,'String'))])]);
		set(gcbo,'String',num2str(f.ripple))
		f.C=filterc(f.filter,f.order,f.fc,f.ripple);
		f.dots=filtdisp(f);
		f.newflag=1;
	case 'frb'
		assignin('base','C',f.C)
		disp('Filter Diff. Eq. Coefficients')
		evalin('base','C')
	case 'freq'
		v=str2num(get(gcbo,'String'));
		f.freq=min([realmax^0.7,max([0,v])]);
		set(gcbo,'String',num2str(f.freq))
		magtriv(f)
	case 'popfildisp'
		v=get(gcbo,'Value');
		if v<=3
			if v ~= f.respdisp(1)
				f.respdisp(1)=v;
				f.dots(1)=drawmag(f);
				magtriv(f)
			end
		else
			if v-3 ~=f.respdisp(2)
				f.respdisp(2)=v-3;
				f.dots(2)=drawphase(f);
				magtriv(f)
			end
		end
	case 'norm'
		f.normalize=get(gcbo,'Value');
	case 'popwin'
		f.window=get(gcbo,'Value');
		windisp(f)
	case 'popkspct'
		f.kspect=get(gcbo,'Value');
		windisp(f)
	case 'f'
		out=f;
	case 'help'
		helpwin('aboutsimdaq')
	otherwise
		disp(['callback ',action,' unknown'])
	end
	set(gcbf,'UserData',f)
	
% Start-ups
elseif (nargin==0 | (nargin==2 & isstr(arg2)))
	f=struct('mode',1, ...
		'fcnname','unitsquare', ...
		'signal',1, ...
		'fs',10, ...
		'N',20, ...
		'start',0, ...
		'warmup',0, ...
		'newflag',1, ...
		'spectmode',1, ...
		'y',zeros(30,1), ...
		'Y',zeros(30,1), ...
		'mean',0, ...
		'filter',3, ...
		'order',3, ...
		'fc',3, ...
		'ripple',0.2929, ...
		'respdisp',[1 1], ...
		'C', [1 1 1], ...
		'freq',3, ...
		'window',1, ...
		'normalize',0, ...
		'kspect',1, ...
		'datax',0,'spectax',0,'trivia',0,'daqclump',zeros(18,0), ...
		'magax',0,'phaseax',0,'poleax',0,'magtriv',0,'dots',[0 0], ...
		'filtclump',zeros(17,0),'winclump',zeros(8,0));
	if (nargin==2)	f.fcnname=arg2;	end
    a=figure('Units','normalized','Position',[0.17 0.07 0.8 0.8], ...
      'Numbertitle','off','Name','SimDAQ','Tag','SimDAQBR549');
	set(0,'ShowHiddenHandles','on')
	wh=findobj('Tag','SimDAQBR549');
	s=length(wh);
	if (s>1)
		set(a,'Name',['SimDAQ #',num2str(s)]);
	end
	set(0,'ShowHiddenHandles','off')
	currdir=pwd;
	homedir=pathto('mainswitch.m');
	if isunix cd('/'), end
	cd(homedir)
	d=dir('fontpref.mat');
	if (length(d))
		load fontpref
		set(a,'DefaultAxesFontName',axfont.FontName, ...
			'DefaultAxesFontUnits',axfont.FontUnits, ...
			'DefaultAxesFontSize',axfont.FontSize, ...
			'DefaultAxesFontWeight',axfont.FontWeight, ...
			'DefaultAxesFontAngle',axfont.FontAngle, ...
			'DefaultUicontrolFontName',uifont.FontName, ...
			'DefaultUicontrolFontUnits',uifont.FontUnits, ...
			'DefaultUicontrolFontSize',uifont.FontSize, ...
			'DefaultUicontrolFontWeight',uifont.FontWeight, ...
			'DefaultUicontrolFontAngle',uifont.FontAngle)
	end
	if isunix cd('/'), end
	cd(currdir)
	color=get(a,'Color');
	c=uicontrol('Parent',a,'Style','text','Units','normalized', ...
	   'Position',[0.015 0.94 0.195 0.05],'BackgroundColor',color, ...
	   'String','Program Mode');
	c=uicontrol('Parent',a,'Style','popupmenu','Units','normalized', ...
	   'Position',[0.015 0.88 0.195 0.05],'BackgroundColor',color, ...
	   'String',['Signal|Filter|Window'], ...
	   'Callback','simdaq popmode','Value',1);
	% display objects for DAQ mode
	f.datax=axes('Parent',a,'Units','normalized','XGrid','on', ...
		'YGrid','on','Position',[0.33 0.56 0.6 0.39]);
	f.spectax=axes('Parent',a,'Units','normalized','XGrid','on', ...
		'YGrid','on','Position',[0.33 0.07 0.6 0.39]);
	popsig=uicontrol('Parent',a,'Style','popupmenu', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.015 0.814 0.118 0.056], ...
	   'String',[f.fcnname,'|unitstep|problem|noisy'], ...
	   'Callback','simdaq popsig','Value',1);
	sigtxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.133 0.814 0.108 0.056],'String','Signal');
	samptxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.015 0.748 0.15 0.056],'String','Sampling');
	fs=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.015 0.682 0.118 0.056],'String',num2str(f.fs), ...
	   'callback','simdaq fs');
	fstxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.133 0.682 0.108 0.056],'String','fs');
	N=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.015 0.616 0.118 0.056],'String',num2str(f.N), ...
	   'callback','simdaq N');
	Ntxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.133 0.616 0.108 0.056],'String','N');
	start=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.015 0.55 0.118 0.056],'String',num2str(f.start), ...
	   'callback','simdaq start');
	starttxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.133 0.55 0.108 0.056],'String','Start');
	warmup=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.015 0.484 0.118 0.056],'String',num2str(f.warmup), ...
	   'callback','simdaq warmup');
	warmuptxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.133 0.484 0.108 0.056],'String','Warmup');
	f.trivia=uicontrol('Parent',a,'Style','text','Units','normalized', ...
	   'Position',[0 0.2 0.241 0.25],'BackgroundColor',color, ...
	   'String',{'Butterworth Filter';'Order 3';'fc = 3';'No Window'});
	popdisp=uicontrol('Parent',a,'Style','popupmenu', ...
	   'Units','normalized','Position',[0.015 0.14 0.118 0.048], ...
	   'String','Amplitude|Power|Complex|Decibel|Reconstruct', ...
	   'Callback','simdaq popdisp','Value',1,'BackgroundColor',color);
	disptxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','Position',[0.133 0.14 0.108 0.048], ...
	   'String','Display','BackgroundColor',color);
	zoom=uicontrol('Parent',a,'Style','checkbox','Units','normalized', ...
		'Position',[0.015 0.09 0.165 0.048],'String','Zoom Control', ...
		'callback','simdaq zoom','BackgroundColor',color);
	rbutton=uicontrol('Parent',a,'Style','pushbutton', ...
	   'Units','normalized','Position',[0.015 0.015 0.15 0.06], ...
	   'String','Return','Callback','simdaq return');
	f.daqclump=[f.datax;f.spectax;sigtxt;popsig;samptxt;fs;fstxt;
		N;Ntxt;start;starttxt;warmup;warmuptxt;zoom;f.trivia;
		disptxt;popdisp;rbutton];
	% display objects for filter design mode
	f.magax=axes('Parent',a,'Units','normalized', ...
		'Position',[0.56 0.56 0.41 0.39],'box','on','Visible','off');
	f.phaseax=axes('Parent',a,'Units','normalized', ...
		'Position',[0.56 0.07 0.41 0.39],'box','on','Visible','off');
	f.poleax=axes('Parent',a,'Units','normalized', ...
		'Position',[0.07 0.07 0.41 0.39],'box','on','Visible','off');
	popfilt=uicontrol('Parent',a,'Style','popupmenu', ...
	   'Units','normalized','Position',[0.015 0.814 0.108 0.056], ...
	   'String',['None|Bessel|Butterworth|Chebychev'], ...
	   'BackgroundColor',color,'Visible','off', ...
	   'Callback','simdaq popfilt','Value',f.filter);
	pftxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','Position',[0.138 0.814 0.108 0.056], ...
	   'BackgroundColor',color,'String','Filter','Visible','off');
	order=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.015 0.748 0.108 0.056],'String',num2str(f.order), ...
	   'callback','simdaq order','Visible','off');
	ordertxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','Position',[0.138 0.748 0.108 0.056], ...
	   'BackgroundColor',color,'String','Order','Visible','off');
	fc=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.015 0.682 0.108 0.056],'String',num2str(f.fc), ...
	   'callback','simdaq fc','Visible','off');
	fctxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','Position',[0.138 0.682 0.108 0.056], ...
	   'BackgroundColor',color,'String','fc','Visible','off');
	rip=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.015 0.616 0.108 0.056],'String',num2str(f.ripple), ...
	   'callback','simdaq ripple','Visible','off','Enable','off');
	riptxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','Position',[0.138 0.616 0.108 0.056], ...
	   'BackgroundColor',color,'String','ripple','Visible','off');
	frb=uicontrol('Parent',a,'Style','pushbutton','Units','normalized', ...
	   'Position',[0.015 0.55 0.108 0.056],'String','Return', ...
	   'callback','simdaq frb','Visible','off');
	freq=uicontrol('Parent',a,'Style','edit','Units','normalized', ...
	   'Position',[0.26 0.9165 0.108 0.056],'String',num2str(f.freq), ...
	   'callback','simdaq freq','Visible','off');
	freqtxt=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','Position',[0.383 0.9165 0.108 0.056], ...
	   'BackgroundColor',color,'String','Check f','Visible','off');
	f.magtriv=uicontrol('Parent',a,'Style','text', ...
	   'Units','normalized','Position',[0.26 0.655 0.21 0.2265], ...
	   'BackgroundColor',color,'String','magtriv','Visible','off');
	fildisptxt=uicontrol('Parent',a,'Style','text','Visible','off', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.27 0.60 0.195 0.05],'String','Filter Display');
	popfildisp=uicontrol('Parent',a,'Style','popupmenu', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.27 0.55 0.195 0.05],'Visible','off', ...
	   'String',['Magnitude|Power|Decibel|Phase Delay|Time Delay'], ...
	   'Callback','simdaq popfildisp','Value',f.respdisp(1));
	f.filtclump=[f.magax;f.phaseax;f.poleax;popfilt;pftxt;
		order;ordertxt;fc;fctxt;rip;riptxt;frb;freq;freqtxt;
		f.magtriv;fildisptxt;popfildisp];
	% display objects for Window mode follow
	normchk=uicontrol('Parent',a,'Style','checkbox','Units','normalized', ...
		'Position',[0.015 0.82 0.195 0.05],'String','Normalize Data', ...
		'callback','simdaq norm','Visible','off');
	wintxt=uicontrol('Parent',a,'Style','text','Visible','off', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.015 0.70 0.195 0.05],'String','Window');
	popwin=uicontrol('Parent',a,'Style','popupmenu', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.015 0.65 0.195 0.05],'Visible','off', ...
	   'String',['None|Bartlett|Hanning|Hamming|Kaiser 2.5'], ...
	   'Callback','simdaq popwin','Value',f.window);
	winsptxt=uicontrol('Parent',a,'Style','text','Visible','off', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.015 0.60 0.195 0.05],'String','Kernel Display');
	popkspect=uicontrol('Parent',a,'Style','popupmenu', ...
	   'Units','normalized','BackgroundColor',color, ...
	   'Position',[0.015 0.55 0.195 0.05],'Visible','off', ...
	   'String',['Magnitude|Power|Decibel'], ...
	   'Callback','simdaq popkspct','Value',1);
	f.winclump=[f.magax;f.phaseax;f.poleax;normchk;wintxt;popwin;
		winsptxt;popkspect];
	b = uimenu('Parent',a,'Label','SimDAQ','Tag','uimenu1');
	c1 = uimenu('Parent',b,'Label','Help','Enable','on', ...
		'Tag','helpmenu','Callback','simdaq help');
	c2 = uimenu('Parent',b,'Label','Visible','Enable','on', ...
		'Callback',['set(gcf,''Handlevisibility'',''on'');', ...
			'set(get(gcbo,''UserData''),''Enable'',''on'');', ...
			'set(gcbo,''Enable'',''off'')'],'Tag','onmenu');
	c3 = uimenu('Parent',b,'Label','Callback','Enable','off', ...
		'Callback',['set(gcf,''Handlevisibility'',''callback'');', ...
			'set(get(gcbo,''UserData''),''Enable'',''on'');', ...
			'set(gcbo,''Enable'',''off'')'],'UserData',c2);
	set(c2,'UserData',c3)
% 	c=uicontrol('Parent',a,'Style','pushbutton','Units','normalized', ...
% 	   'Position',[0.18 0.015 0.05 0.06],'String','f', ...
% 	   'Callback','f=simdaq(''f'')');

	f.C=filterc(f.filter,f.order,f.fc,f.ripple);
	[f.y,f.Y,f.mean]=daqdisp(f);
	f.newflag=0;
	set(a,'UserData',f)
	set(gcf,'HandleVisibility','callback')
else
	error('Argument Error!')
end

function [yout,Yout,meanout]=daqdisp(f);
set(get(f.datax,'children'),'visible','on')
if f.newflag
	t=f.start+[0:f.N-1]'/f.fs;
	tdisp=linspace(f.start-f.warmup,f.start+f.N/f.fs,501)';
	tg=[t;tdisp];		% ode45 requires sorted time basis
	[ts,in]=sort(tg);
	if any(diff(ts)==0)
		tdisp=tdisp-eps^0.75/f.fs;	% offset if necessary
		tg=[t;tdisp];	[ts,in]=sort(tg);	% and re-sort
	end
	[is,jn]=sort(in);	% set up sort reversal index jn
	fcnnames={f.fcnname;'unitstep';'problem';'noisy'};
	fcnname=fcnnames{f.signal};
	if f.filter==1	% no filter
		f.y=feval(fcnname,t);
		ys=feval(fcnname,ts);
	else			% apply filter equation C
		r0=[feval(fcnname,ts(1));zeros(f.order-1,1)];
		B=[-f.C(end:-1:2), 1]/f.C(1);
		[td,r]=ode45('simdaq',ts,r0,[],fcnname,B);
		ys=r(:,1);
		f.y=ys(jn(1:f.N));	% extract sampled points
	end
	F=feval(fcnname,ts);
	axes(f.datax), cla
	Fh=plot(ts,F,'k','LineWidth',1.5);	hold on
	yh=plot(ts,ys,'g');
	if f.N<50
		stem(t,f.y,'bo')
	else
		plot(t,f.y,'bo')
	end
	grid on
	legend([Fh,yh],fcnname,'Response')
	title(['Filter Response to ',fcnname,', and Samples'])
	xlabel('Time'),	ylabel('Input and Output')
end
y=f.y;
meany=mean(f.y);
if f.normalize
	y=y-meany;
end
[winfac,winname]=window(f.window,f.N);
y=y.*winfac';
Y=fft(y);
frange=[0:f.N-1]/f.N*f.fs;
if mod(f.N,2)	mid=(f.N+1)/2; else mid=f.N/2+1;	end
n=1:mid;
axes(f.spectax),	cla
switch(f.spectmode)
case 1
	if f.N<100
		stem(frange(n),2*abs(Y(n))/f.N)
	else
		plot(frange(n),2*abs(Y(n))/f.N)
	end
	title('Amplitude Spectrum'), xlabel('Frequency'),	ylabel('A')
case 2
	if f.N<100
		stem(frange(n),2*Y(n).*conj(Y(n))/f.N^2)
	else
		plot(frange(n),2*Y(n).*conj(Y(n))/f.N^2)
	end
	title('Power Spectrum'),	xlabel('Frequency'),	ylabel('A')
case 3
	if f.N<100
		plot([1 1]*frange(1),[0 real(Y(1))/f.N],'r'), hold on
		plot([1 1]*frange(1),[0 imag(Y(1))/f.N],'b')
		stem(frange(n),2*real(Y(n))/f.N,'r')
		stem(frange(n),2*imag(Y(n))/f.N,'b')
	else
		plot(frange(n),2*real(Y(n))/f.N,'r'),	hold on
		plot(frange(n),2*imag(Y(n))/f.N,'b')
	end
	legend('real','imag',0)
	title('Complex Spectrum'),	xlabel('Frequency'),	ylabel('Y')
case 4
	plot(frange(n),20*log10(sqrt(2)*abs(Y(n)/f.N)))
	title('Decibel Spectrum'),	xlabel('Frequency'),	ylabel('dB')
case 5
	t2=[0:4*f.N-1]/f.fs/4;
	Y2=[Y(n);zeros(3*f.N,1);Y(mid+1:f.N)];
	y2=real(ifft(Y2))*4;
	plot(t2,y2)
	title('Reconstructed Signal'),	xlabel('Time'),	ylabel('Output')
end
grid on
str={[filtname(f.filter),' Filter Applied']};
if f.filter>1
	str=cat(1,str,{['order = ',num2str(f.order)];
		['fc = ',num2str(f.fc)]});
end
if f.filter==4
	str=cat(1,str,{['ripple = ',num2str(f.ripple)]});
end
str=cat(1,str,{'';['Sample Mean = ',num2str(meany)]});
if f.normalize	str=cat(1,str,{'Mean Subtracted'});	end
str=cat(1,str,{'';[winname,' Window Applied']});
set(f.trivia,'String',str)
if nargout 
	yout=f.y;
	Yout=Y;
	meanout=meany;
end

function dots=filtdisp(f)
dots=[0 0];
dots(1)=drawmag(f);
dots(2)=drawphase(f);
f.dots=dots;
axes(f.poleax),	cla
if f.filter>1			% Poles Display
	r=roots(f.C)/2/pi;
	if f.order==1
		plot(real(r(1)),0,'rx')
	else
		plot(r,'rx')
	end
	xl=min(real(r));
	set(f.poleax,'XLim',[1.2*xl -0.2*xl])
	title([filtname(f.filter),' Filter Poles in the S-Plane'])
else
	title('No Filter')
end
grid on
xlabel('Real (1/sec)'),	ylabel('Frequency (Hz)')
magtriv(f)

function h=drawmag(f)
axes(f.magax),	cla
if f.filter>1
	switch f.respdisp(1)
	case 1	% Magnitude
		frange=linspace(0,3*f.fc,501);
		wrange=2*pi*frange;
		Hrange=1./polyval(f.C,j*wrange);
		plot(frange,abs(Hrange),'b')
		title([filtname(f.filter),' Filter Magnitude Attenuation'])
		ylabel('Attenuation Factor')
	case 2	% Power
		frange=exp(linspace(log(f.fc/10),log(f.fc*10),501));
		wrange=2*pi*frange;
		Hrange=1./polyval(f.C,j*wrange);
		loglog(frange,abs(Hrange).^2,'b')
		title([filtname(f.filter),' Filter Power Attenuation'])
		ylabel('Attenuation Factor')
	case 3	% Decibels
		frange=exp(linspace(log(f.fc/10),log(f.fc*10),501));
		wrange=2*pi*frange;
		Hrange=1./polyval(f.C,j*wrange);
		semilogx(frange,20*log10(abs(Hrange)),'b')
		title([filtname(f.filter),' Decibel Power Attenuation'])
		ylabel('Attenuation (Decibels)')
	end
else		% No Filter display
	if f.respdisp(1)==3
		plot([0 3*f.fc],[0 0])
	else
		plot([0 3*f.fc],[1 1])
	end
	title('No Filter - Unit Attenuation')
end
xlabel('Frequency')
h=line('marker','o','color','b','xdata',[],'ydata',[]);

function h=drawphase(f)
axes(f.phaseax),	cla
if f.filter>1
	frange=linspace(0,3*f.fc,501);
	frange=frange(2:end);
	wrange=2*pi*frange;
	Hrange=1./polyval(f.C,j*wrange);
	phi=unwrap(angle(Hrange));
	switch f.respdisp(2)
	case 1	% Phase Delay
		plot(frange,phi*180/pi,'b')
		title([filtname(f.filter),' Filter Phase Delay'])
		ylabel(['Delay (deg)'])
	case 2	% Time Delay
		plot(frange,phi./wrange,'b')
		title([filtname(f.filter),' Filter Time Delay'])
		ylabel('Delay (sec)')
	end
else		% No Filter display
	plot([0 3*f.fc],[0 0])
	title('No Filter - zero Delay')
end
xlabel('Frequency')
h=line('marker','o','color','b','xdata',[],'ydata',[]);

function magtriv(f)
w=f.freq*2*pi;	wc=f.fc*2*pi;
if f.filter>1	% Filter responses
	delay=f.C(end-1)/f.C(end);
	H=1/polyval(f.C,j*w);
	phi=angle(H);
	if w>pi/2/delay		% potential angle jumps
		frange=exp(linspace(log(f.fc/10),log(f.freq),201));
		wrange=2*pi*frange;
		Hrange=1./polyval(f.C,j*wrange);
		phirange=unwrap(angle(Hrange));
		phi=phirange(201);
	end
else	% No Filter response
	delay=0;
	H=1;
	phi=0;
end
switch f.respdisp(1)
case 1
	y=abs(H);
	str={'Magnitude Attenuation';
		['M(',num2str(f.freq),') = ',num2str(y)]};
case 2
	y=H*conj(H);
	str={'Power Attenuation';
		['P(',num2str(f.freq),') = ',num2str(y)]};
case 3
	y=20*log10(abs(H));
	str={'Power Attenuation';
		['dB(',num2str(f.freq),') = ',num2str(y)]};
end
set(f.dots(1),'XData',f.freq,'YData',y)
switch f.respdisp(2)
case 1
	y=phi*180/pi;
	str=cat(1,str,{'';'Phase Delay';['phi = ',num2str(-y),' deg']});
case 2
	if w==0
		y=-delay;
	else
		y=phi/w;
	end
	str=cat(1,str,{'';'Time Delay';['tau = ',num2str(-y),' sec']});
end
set(f.dots(2),'XData',f.freq,'YData',y)
str=cat(1,str,{['base delay = ',num2str(delay),' sec']});
if f.filter==4
	epsi=sqrt(1/(1-f.ripple)^2-1);
	str=cat(1,str,{'';['Chebychev eps = 'num2str(epsi)]});
end
set(f.magtriv,'String',str)

function windisp(f)
[w,name]=window(f.window,f.N);
axes(f.magax),	cla
plot([1:f.N],w,'b-')
title([name,' Window Form']),xlabel('Sample Number')
wp=[w,zeros(1,3*f.N)];
W=fft(wp);
switch (f.kspect)
case 1		% Magnitude Spectrum
	sptype='Magnitude';
	WD=abs(W)/f.N;
case 2		% Power Spectrum
	sptype='Power';
	WD=W.*conj(W)/f.N^2;
case 3		% Decibel Spectrum
	sptype='Decibel';
	WD=20*log10(abs(W)/f.N);
end
fdom=(0:2*f.N)/f.N/4*f.fs;
axes(f.poleax), cla
nearend=min([17,length(fdom)]);
idx1=nearend:-1:2;	idx2=1:nearend;
plot([-fdom(idx1),fdom(idx2)],[WD(idx1),WD(idx2)],'b-')
xlabel('Relative Frequency'),ylabel([sptype,' Spectrum'])
title([name,' Window Spectrum Kernel - Near Field'])
axes(f.phaseax), cla
idx1=2*f.N+1:-1:2;	idx2=1:2*f.N+1;
plot([-fdom(idx1),fdom(idx2)],[WD(idx1),WD(idx2)],'b-')
xlabel('Relative Frequency'),ylabel([sptype,' Spectrum'])
title([name,' Window Spectrum Kernel - Whole Field'])
if f.kspect==3
	set(f.poleax,'YLim',[-100 0])
	set(f.phaseax,'YLim',[-100 0])
end

function C=filterc(type,order,fc,ripple)
wc=fc*2*pi;
switch (type)
case 1	% No Filter
	C=[];
case 2	% Bessel Filter
	Dkn2=[1];	Dkn1=[1 1];
	if order==1
		Dk=[1 1];
	else	% Build Bessel Polynomial
		Dkn2=[1];	Dkn1=[1 1];
		for k=2:order	
			Dk=[0,(2*k-1)*Dkn1] + [Dkn2,0,0];
			Dkn2=Dkn1;
			Dkn1=Dk;
		end
	end
	Ck=Dk/Dk(end);	% Unit DC Gain
	% set up search for base wc
	gstr=['abs(abs(1./polyval([',num2str(Ck,'%23.15e'),'],j*w))-sqrt(0.5))'];
	Hmagerr=inline(gstr,'w');
	bwc=fmin(Hmagerr,0.9,10);		% search
	C=Ck;
	for k=1:order		% transform frequency
		C(k)=Ck(k)/(wc/bwc)^(order-k+1);
	end
case 3	% Butterworth Filter
	if mod(order,2)			% odd order
		k=0:(order-1)/2;
		pkr=cos(pi*(1-k/order));	% pole positions
		pki=sin(pi*(1-k/order));
		Ck=[1 1];
		for k=2:(order+1)/2
			Ck=conv(Ck,[1 -2*pkr(k) pkr(k)^2+pki(k)^2]);
		end
	else					% even order
		k=0:order/2-1;
		pkr=cos(pi*(1-0.5/order-k/order));	% pole positions
		pki=sin(pi*(1-0.5/order-k/order));
		Ck=[1 -2*pkr(1) pkr(1)^2+pki(1)^2];
		for k=2:order/2
			Ck=conv(Ck,[1 -2*pkr(k) pkr(k)^2+pki(k)^2]);
		end
	end
	C=ones(1,order+1);
	for k=1:order		% transform frequency
		C(k)=Ck(k)/wc^(order-k+1);
	end
case 4 % Chebychev Filter
	eps=sqrt(1/(1-ripple)^2-1);
	rads=sqrt(1/eps^2+1)+1/eps;
	a=(rads^(1/order) - rads^(-1/order))/2;
	b=(rads^(1/order) + rads^(-1/order))/2;
	if mod(order,2)			% odd order
		k=0:(order-1)/2;
		pkr=a*cos(pi*(1-k/order));	% pole positions
		pki=b*sin(pi*(1-k/order));
		Ck=[1 a];
		for k=2:(order+1)/2
			Ck=conv(Ck,[1 -2*pkr(k) pkr(k)^2+pki(k)^2]);
		end
		Ck=Ck/Ck(end);		% Unit DC Gain
	else				% even order
		k=0:order/2-1;
		pkr=a*cos(pi*(1-0.5/order-k/order));	% pole positions
		pki=b*sin(pi*(1-0.5/order-k/order));
		Ck=[1 -2*pkr(1) pkr(1)^2+pki(1)^2];
		for k=2:order/2
			Ck=conv(Ck,[1 -2*pkr(k) pkr(k)^2+pki(k)^2]);
		end
		Ck=Ck/Ck(end)*sqrt(1+eps^2);	% Gain adjustment
	end
	C=Ck;
	for k=1:order		% transform frequency
		C(k)=Ck(k)/wc^(order-k+1);
	end
end

function name=filtname(type)
names={'No';'Bessel';'Butterworth';'Chebychev'};
name=names{type};

function [w,name]=window(type,N)
switch (type)
case 1	% no window
	name='No';
	w=ones(1,N);
case 2	% Bartlett window
	name='Bartlett';
	w=2*[0:N/2]/N;
	if rem(N,2)	% odd length
		w=[w w((N+1)/2:-1:2)];
	else		% even length
		w=[w w(N/2:-1:2)];
	end
case 3	% Hanning window
	name='Hanning';
	w=0.5*(1-cos(2*pi*(0:N-1)/N));
case 4	% Hamming window
	name='Hamming';
	w=0.54-0.46*cos(2*pi*(0:N-1)/N);
case 5	% Kaiser window using alpha=2.5
	name='Kaiser 2.5';
	m=N/2;	t=[0:N-1];
	w=besseli(0,2.5*pi*sqrt(1-((t-m)/m).^2))/besseli(0,2.5*pi);
end

function s = unitsquare(t)
tmp = mod(t+eps^0.75,1);
nodd = (tmp < 0.5);
s = 2*nodd-1;

function y=unitstep(t)
y = t>0;

function y=problem(t)
y=10*sin(2*pi*11.3*t) + 0.2*cos(2*pi*15*t);

function y=noisy(t)
y=10*cos(2*pi*10*t) + abs(5*sin(2*pi*61*t));

function dest=pathto(fname)
r=which(fname);
[tok1,r]=strtok(r,filesep);
dest=tok1;
[tok2,r]=strtok(r,filesep);
while ~isempty(r)
	tok1=tok2;
	dest=[dest,filesep,tok1];
	[tok2,r]=strtok(r,filesep);
end

% by Tim Conover Copyright 2000
